==> v0.4.1.tar.gz: netauth-0.4.1/internal/tree/hooks/entity-capabilities.go <==
package hooks

import (
	"github.com/netauth/netauth/internal/startup"
	"github.com/netauth/netauth/internal/tree"

	pb "github.com/netauth/protocol"
)

// ManageEntityCapabilities is a configurable runtime hook that adds
// or removes capabilities as configured.  Two convenience
// constructors exist to return hooks in either mode.
type ManageEntityCapabilities struct {
	tree.BaseHook
	mode bool
}

// Run modifies the stored capabilities on an entity depending on the
// value of the mode variable.  When the mode is set to true, any
// capabilities stored in de will be copied to e if they are not
// already present.  In false capabilities will be subtracted.
func (mec *ManageEntityCapabilities) Run(e, de *pb.Entity) error {
	if de.Meta == nil || len(de.Meta.Capabilities) == 0 {
		return tree.ErrUnknownCapability
	}
	for _, cap := range de.Meta.Capabilities {
		if mec.mode {
			// Add mode
			e.Meta.Capabilities = addCapability(cap, e.Meta.Capabilities)
		} else {
			// Del mode
			e.Meta.Capabilities = delCapability(cap, e.Meta.Capabilities)
		}
	}
	return nil
}

func init() {
	startup.RegisterCallback(entityCapabilitiesCB)
}

func entityCapabilitiesCB() {
	tree.RegisterEntityHookConstructor("set-entity-capability", NewSetEntityCapability)
	tree.RegisterEntityHookConstructor("remove-entity-capability", NewRemoveEntityCapability)
}

// NewSetEntityCapability returns a ManageEntityCapability hook
// pre-configured into the additive mode.
func NewSetEntityCapability(c tree.RefContext) (tree.EntityHook, error) {
	return &ManageEntityCapabilities{tree.NewBaseHook("set-entity-capability", 50), true}, nil
}

// NewRemoveEntityCapability returns a ManageEntityCapability hook
// pre-configured into the subtractive mode.s
func NewRemoveEntityCapability(c tree.RefContext) (tree.EntityHook, error) {
	return &ManageEntityCapabilities{tree.NewBaseHook("remove-entity-capability", 50), false}, nil
}

==> v0.6.1.tar.gz: netauth-0.6.1/internal/tree/hooks/entity-capabilities.go <==
package hooks

import (
	"context"

	"github.com/netauth/netauth/internal/startup"
	"github.com/netauth/netauth/internal/tree"

	pb "github.com/netauth/protocol"
)

// ManageEntityCapabilities is a configurable runtime hook that adds
// or removes capabilities as configured.  Two convenience
// constructors exist to return hooks in either mode.
type ManageEntityCapabilities struct {
	tree.BaseHook
	mode bool
}

// Run modifies the stored capabilities on an entity depending on the
// value of the mode variable.  When the mode is set to true, any
// capabilities stored in de will be copied to e if they are not
// already present.  In false capabilities will be subtracted.
func (mec *ManageEntityCapabilities) Run(_ context.Context, e, de *pb.Entity) error {
	if de.Meta == nil || len(de.Meta.Capabilities) == 0 {
		return tree.ErrUnknownCapability
	}
	for _, cap := range de.Meta.Capabilities {
		if mec.mode {
			// Add mode
			e.Meta.Capabilities = addCapability(cap, e.Meta.Capabilities)
		} else {
			// Del mode
			e.Meta.Capabilities = delCapability(cap, e.Meta.Capabilities)
		}
	}
	return nil
}

func init() {
	startup.RegisterCallback(entityCapabilitiesCB)
}

func entityCapabilitiesCB() {
	tree.RegisterEntityHookConstructor("set-entity-capability", NewSetEntityCapability)
	tree.RegisterEntityHookConstructor("remove-entity-capability", NewRemoveEntityCapability)
}

// NewSetEntityCapability returns a ManageEntityCapability hook
// pre-configured into the additive mode.
func NewSetEntityCapability(opts ...tree.HookOption) (tree.EntityHook, error) {
	opts = append([]tree.HookOption{
		tree.WithHookName("set-entity-capability"),
		tree.WithHookPriority(50),
	}, opts...)

	return &ManageEntityCapabilities{tree.NewBaseHook(opts...), true}, nil
}

// NewRemoveEntityCapability returns a ManageEntityCapability hook
// pre-configured into the subtractive mode.s
func NewRemoveEntityCapability(opts ...tree.HookOption) (tree.EntityHook, error) {
	opts = append([]tree.HookOption{
		tree.WithHookName("remove-entity-capability"),
		tree.WithHookPriority(50),
	}, opts...)

	return &ManageEntityCapabilities{tree.NewBaseHook(opts...), false}, nil
}
